package org.nakedobjects.nof.util.memento;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;


public abstract class AbstractTransferableReader implements TransferableReader {

    public Transferable readObject() {
        String className = readString();
        try {
            Class c = Class.forName(className);
            Constructor constructor = c.getConstructor(new Class[] { TransferableReader.class });
            Transferable object = (Transferable) constructor.newInstance(new Object[] { this });
            return object;
        } catch (ClassNotFoundException e) {
            throw new TransferableException("Could not find class " + className, e);
        } catch (InstantiationException e) {
            throw new TransferableException("Failed to instantiate instance of " + className, e);
        } catch (IllegalAccessException e) {
            throw new TransferableException(e);
        } catch (SecurityException e) {
            throw new TransferableException(e);
        } catch (NoSuchMethodException e) {
            throw new TransferableException("No constructor taking a TransferableReader in " + className, e);
        } catch (IllegalArgumentException e) {
            throw new TransferableException(e);
        } catch (InvocationTargetException e) {
            throw new TransferableException(e);
        }
    }

}
// Copyright (c) Naked Objects Group Ltd.
