package org.nakedobjects.nof.util.xmlsnapshot;

import org.apache.crimson.tree.ElementNode2;
import org.nakedobjects.noa.adapter.NakedObject;
import org.w3c.dom.Element;


/**
 * Represents a place in the graph to be navigated; really just wraps an object and an XML Element in its XML
 * document. Also provides the capability to extract the corresponding XSD element (associated with each XML
 * element).
 * 
 * The XML element (its children) is mutated as the graph of objects is navigated.
 */
final class Place {
    private final NakedObject object;
    private final Element element;

    Place(final NakedObject object, final Element element) {
        this.object = object;
        this.element = element;
    }

    public Element getXmlElement() {
        return element;
    }

    public NakedObject getObject() {
        return object;
    }

    public Element getXsdElement() {
        if (!(element instanceof ElementNode2)) {
            return null;
        }
        ElementNode2 e = (ElementNode2) element;
        Object o = e.getUserObject();
        if (o == null || !(o instanceof Element)) {
            return null;
        }
        return (Element) o;
    }

    // TODO: smelly; where should this responsibility lie?
    static void setXsdElement(final Element element, final Element xsElement) {
        if (!(element instanceof ElementNode2)) {
            return;
        }
        ElementNode2 e = (ElementNode2) element;
        e.setUserObject(xsElement);
    }

}
// Copyright (c) Naked Objects Group Ltd.
